@@ -28,7 +28,6 @@ import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-
 import java.sql.SQLException;
 
 /**
@@ -46,7 +45,7 @@ import java.sql.SQLException;
  * class is new in the JDBC 2.0 API.
  * 
  * @author Mark Matthews
- * @version $Id: Blob.java 6437 2007-05-24 20:17:01Z mmatthews $
+ * @version $Id: Blob.java 6734 2008-02-20 17:13:12Z mmatthews $
  */
 public class Blob implements java.sql.Blob, OutputStreamWatcher {
 
@@ -239,8 +238,11 @@ public class Blob implements java.sql.Blob, OutputStreamWatcher {
 		try {
 			bytesOut.write(bytes, offset, length);
 		} catch (IOException ioEx) {
-			throw SQLError.createSQLException(Messages.getString("Blob.1"), //$NON-NLS-1$
+			SQLException sqlEx = SQLError.createSQLException(Messages.getString("Blob.1"), //$NON-NLS-1$
 					SQLError.SQL_STATE_GENERAL_ERROR);
+			sqlEx.initCause(ioEx);
+			
+			throw sqlEx;
 		} finally {
 			try {
 				bytesOut.close();
@@ -294,7 +296,7 @@ public class Blob implements java.sql.Blob, OutputStreamWatcher {
 	public synchronized void truncate(long len) throws SQLException {
 		checkClosed();
 		
-		if (len < 1) {
+		if (len < 0) {
 			throw SQLError.createSQLException("\"len\" argument can not be < 1.", 
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
