@@ -180,7 +180,8 @@ public class BufferRow extends ResultSetRow {
 		if (index == 0) {
 			this.lastRequestedIndex = 0;
 			this.lastRequestedPos = this.homePosition;
-
+			this.rowFromServer.setPosition(this.homePosition);
+			
 			return 0;
 		}
 
@@ -636,12 +637,14 @@ public class BufferRow extends ResultSetRow {
 		throw new OperationNotSupportedException();
 	}
 
-	public void setMetadata(Field[] f) throws SQLException {
+	public ResultSetRow setMetadata(Field[] f) throws SQLException {
 		super.setMetadata(f);
 
 		if (this.isBinaryEncoded) {
 			setupIsNullBitmask();
 		}
+		
+		return this;
 	}
 
 	/**
@@ -684,7 +687,7 @@ public class BufferRow extends ResultSetRow {
 	}
 
 	public Date getDateFast(int columnIndex, ConnectionImpl conn,
-			ResultSetImpl rs) throws SQLException {
+			ResultSetImpl rs, Calendar targetCalendar) throws SQLException {
 		if (isNull(columnIndex)) {
 			return null;
 		}
@@ -696,7 +699,7 @@ public class BufferRow extends ResultSetRow {
 		int offset = this.rowFromServer.getPosition();
 
 		return getDateFast(columnIndex, this.rowFromServer.getByteBuffer(),
-				offset, (int)length, conn, rs);
+				offset, (int)length, conn, rs, targetCalendar);
 	}
 
 	public java.sql.Date getNativeDate(int columnIndex, ConnectionImpl conn,
