@@ -25,13 +25,9 @@ package com.mysql.jdbc;
 import java.io.InputStream;
 import java.io.Reader;
 import java.io.UnsupportedEncodingException;
-
 import java.lang.reflect.Constructor;
-import java.lang.reflect.InvocationTargetException;
 import java.math.BigDecimal;
-
 import java.net.URL;
-
 import java.sql.Array;
 import java.sql.Blob;
 import java.sql.Clob;
@@ -42,16 +38,12 @@ import java.sql.SQLException;
 import java.sql.Time;
 import java.sql.Timestamp;
 import java.sql.Types;
-
 import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.Properties;
-
-import com.mysql.jdbc.exceptions.NotYetImplementedException;
 
 /**
  * Representation of stored procedures for JDBC
@@ -609,10 +601,19 @@ public class CallableStatement extends PreparedStatement implements
 
 		this.callingStoredFunction = isFunctionCall;
 
-		determineParameterTypes();
-		generateParameterMap();
-		
-		if (this.callingStoredFunction) {
+		if (!this.callingStoredFunction) {
+			if (!StringUtils.startsWithIgnoreCaseAndWs(sql, "CALL")) {
+				// not really a stored procedure call
+				fakeParameterTypes(false);
+			} else {
+				determineParameterTypes();
+			}
+			
+			generateParameterMap();
+		} else {
+			determineParameterTypes();
+			generateParameterMap();
+			
 			this.parameterCount += 1;
 		}
 	}
@@ -722,7 +723,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * 
 	 * @throws SQLException if we can't build the metadata.
 	 */
-	private void fakeParameterTypes() throws SQLException {
+	private void fakeParameterTypes(boolean isReallyProcedure) throws SQLException {
 		Field[] fields = new Field[13];
 
 		fields[0] = new Field("", "PROCEDURE_CAT", Types.CHAR, 0);
@@ -739,12 +740,12 @@ public class CallableStatement extends PreparedStatement implements
 		fields[11] = new Field("", "NULLABLE", Types.SMALLINT, 0);
 		fields[12] = new Field("", "REMARKS", Types.CHAR, 0);
 
-		String procName = extractProcedureName();
+		String procName = isReallyProcedure ? extractProcedureName() : null;
 
 		byte[] procNameAsBytes = null;
 
 		try {
-			procNameAsBytes = procName.getBytes("UTF-8");
+			procNameAsBytes = procName == null ? null : procName.getBytes("UTF-8");
 		} catch (UnsupportedEncodingException ueEx) {
 			procNameAsBytes = StringUtils.s2b(procName, this.connection);
 		}
@@ -787,7 +788,7 @@ public class CallableStatement extends PreparedStatement implements
 	
 	private void determineParameterTypes() throws SQLException {
 		if (this.connection.getNoAccessToProcedureBodies()) {
-			fakeParameterTypes();
+			fakeParameterTypes(true);
 			
 			return;
 		}
