@@ -33,7 +33,7 @@ import java.util.regex.PatternSyntaxException;
  * Field is a class used to describe fields in a ResultSet
  *
  * @author Mark Matthews
- * @version $Id: Field.java 6577 2007-09-07 16:12:04Z mmatthews $
+ * @version $Id: Field.java 6643 2007-10-23 17:47:15Z mmatthews $
  */
 public class Field {
 
@@ -416,6 +416,12 @@ public class Field {
 		return this.charsetName;
 	}
 
+	public void setCharacterSet(String javaEncodingName) throws SQLException {
+		this.charsetName = javaEncodingName;
+		this.charsetIndex = CharsetMapping
+				.getCharsetIndexForMysqlEncodingName(javaEncodingName);
+	}
+	
 	public synchronized String getCollation() throws SQLException {
 		if (this.collationName == null) {
 			if (this.connection != null) {
@@ -938,7 +944,9 @@ public class Field {
 	public void setConnection(ConnectionImpl conn) {
 		this.connection = conn;
 
-		this.charsetName = this.connection.getEncoding();
+		if (this.charsetName == null || this.charsetIndex == 0) {
+			this.charsetName = this.connection.getEncoding();
+		}
 	}
 
 	void setMysqlType(int type) {
