@@ -25,8 +25,6 @@ package com.mysql.jdbc;
 import java.sql.SQLException;
 import java.sql.Types;
 
-import com.mysql.jdbc.exceptions.NotYetImplementedException;
-
 /**
  * A ResultSetMetaData object can be used to find out about the types and
  * properties of the columns in a ResultSet
@@ -373,7 +371,10 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 
 		case MysqlDefs.FIELD_TYPE_SET:
 			return "SET"; //$NON-NLS-1$
-
+			
+		case MysqlDefs.FIELD_TYPE_GEOMETRY:
+			return "GEOMETRY"; //$NON-NLS-1$
+			
 		default:
 			return "UNKNOWN"; //$NON-NLS-1$
 		}
