@@ -25,15 +25,17 @@
 package com.mysql.jdbc;
 
 import java.sql.SQLException;
+import java.sql.Statement;
 
-import com.mysql.jdbc.profiler.ProfileEventSink;
 import com.mysql.jdbc.profiler.ProfilerEvent;
+import com.mysql.jdbc.profiler.ProfilerEventHandler;
+import com.mysql.jdbc.profiler.ProfilerEventHandlerFactory;
 
 /**
  * Allows streaming of MySQL data.
  * 
  * @author dgan
- * @version $Id: RowDataDynamic.java 6454 2007-06-21 17:03:53Z mmatthews $
+ * @version $Id: RowDataDynamic.java 6706 2008-01-04 02:40:19Z mmatthews $
  */
 public class RowDataDynamic implements RowData {
 
@@ -68,6 +70,8 @@ public class RowDataDynamic implements RowData {
 
 	private boolean useBufferRowExplicit;
 
+	private boolean moreResultsExisted;
+
 	/**
 	 * Creates a new RowDataDynamic object.
 	 * 
@@ -199,7 +203,7 @@ public class RowDataDynamic implements RowData {
 				if (conn.getUseUsageAdvisor()) {
 					if (hadMore) {
 
-						ProfileEventSink eventSink = ProfileEventSink
+						ProfilerEventHandler eventSink = ProfilerEventHandlerFactory
 						.getInstance(conn);
 
 						eventSink
@@ -387,7 +391,7 @@ public class RowDataDynamic implements RowData {
 
 		nextRecord();
 
-		if (this.nextRow == null && !this.streamerClosed) {
+		if (this.nextRow == null && !this.streamerClosed && !this.moreResultsExisted) {
 			this.io.closeStreamer(this);
 			this.streamerClosed = true;
 		}
@@ -413,7 +417,8 @@ public class RowDataDynamic implements RowData {
 
 				if (this.nextRow == null) {
 					this.isAtEnd = true;
-					
+					this.moreResultsExisted = this.io.tackOnMoreStreamingResults(this.owner);
+
 					if (this.index == -1) {
 						this.wasEmpty = true;
 					}
@@ -435,10 +440,13 @@ public class RowDataDynamic implements RowData {
 			exceptionMessage += Messages.getString("RowDataDynamic.7"); //$NON-NLS-1$
 			exceptionMessage += Util.stackTraceToString(ex);
 
-			throw new java.sql.SQLException(
+			SQLException sqlEx = SQLError.createSQLException(
 					Messages.getString("RowDataDynamic.8") //$NON-NLS-1$
 							+ exceptionType
 							+ Messages.getString("RowDataDynamic.9") + exceptionMessage, SQLError.SQL_STATE_GENERAL_ERROR); //$NON-NLS-1$
+			sqlEx.initCause(ex);
+			
+			throw sqlEx;
 		}
 	}
 
