@@ -24,22 +24,14 @@
  */
 package com.mysql.jdbc;
 
-import com.mysql.jdbc.exceptions.MySQLStatementCancelledException;
-import com.mysql.jdbc.exceptions.MySQLTimeoutException;
-import com.mysql.jdbc.profiler.ProfileEventSink;
-import com.mysql.jdbc.profiler.ProfilerEvent;
-
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.Reader;
 import java.io.UnsupportedEncodingException;
-
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.math.BigDecimal;
-
 import java.net.URL;
-
 import java.sql.Array;
 import java.sql.Blob;
 import java.sql.Clob;
@@ -51,12 +43,16 @@ import java.sql.SQLException;
 import java.sql.Time;
 import java.sql.Timestamp;
 import java.sql.Types;
-
 import java.util.ArrayList;
 import java.util.Calendar;
-import java.util.Properties;
+import java.util.GregorianCalendar;
 import java.util.TimeZone;
 
+import com.mysql.jdbc.exceptions.MySQLStatementCancelledException;
+import com.mysql.jdbc.exceptions.MySQLTimeoutException;
+import com.mysql.jdbc.profiler.ProfilerEvent;
+import com.mysql.jdbc.profiler.ProfilerEventHandlerFactory;
+
 /**
  * JDBC Interface for MySQL-4.1 and newer server-side PreparedStatements.
  * 
@@ -208,6 +204,51 @@ public class ServerPreparedStatement extends PreparedStatement {
 				}
 			}
 		}
+		
+		long getBoundLength() {
+			if (isNull) {
+				return 0;
+			}
+			
+			if (isLongData) {
+				return bindLength;
+			}
+
+			switch (bufferType) {
+
+			case MysqlDefs.FIELD_TYPE_TINY:
+				return 1;
+			case MysqlDefs.FIELD_TYPE_SHORT:
+				return 2;
+			case MysqlDefs.FIELD_TYPE_LONG:
+				return 4;
+			case MysqlDefs.FIELD_TYPE_LONGLONG:
+				return 8;
+			case MysqlDefs.FIELD_TYPE_FLOAT:
+				return 4;
+			case MysqlDefs.FIELD_TYPE_DOUBLE:
+				return 8;
+			case MysqlDefs.FIELD_TYPE_TIME:
+				return 9;
+			case MysqlDefs.FIELD_TYPE_DATE:
+				return 7;
+			case MysqlDefs.FIELD_TYPE_DATETIME:
+			case MysqlDefs.FIELD_TYPE_TIMESTAMP:
+				return 11;
+			case MysqlDefs.FIELD_TYPE_VAR_STRING:
+			case MysqlDefs.FIELD_TYPE_STRING:
+			case MysqlDefs.FIELD_TYPE_VARCHAR:
+			case MysqlDefs.FIELD_TYPE_DECIMAL:
+			case MysqlDefs.FIELD_TYPE_NEW_DECIMAL:
+				if (value instanceof byte[]) {
+					return ((byte[]) value).length;
+				} else {
+					return ((String) value).length();
+				}
+			default: 
+				return 0;
+			}
+		}
 	}
 
 	/* 1 (length) + 2 (year) + 1 (month) + 1 (day) */
@@ -387,8 +428,11 @@ public class ServerPreparedStatement extends PreparedStatement {
 		} catch (Exception ex) {
 			realClose(false, true);
 
-			throw SQLError.createSQLException(ex.toString(),
+			SQLException sqlEx = SQLError.createSQLException(ex.toString(),
 					SQLError.SQL_STATE_GENERAL_ERROR);
+			sqlEx.initCause(ex);
+			
+			throw sqlEx;
 		}
 		
 		setResultSetType(resultSetType);
@@ -529,15 +573,23 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 	private boolean useAutoSlowLog;
 
+	private Calendar serverTzCalendar;
+
+	private Calendar defaultTzCalendar;
+
 	protected void setClosed(boolean flag) {
 		this.isClosed = flag;
 	}
+	
 	/**
 	 * @see java.sql.Statement#close()
 	 */
-	public void close() throws SQLException {
-		if (this.isCached) {
+	public synchronized void close() throws SQLException {
+		if (this.isCached && !this.isClosed) {
+			clearParameters();
+			
 			this.isClosed = true;
+			
 			this.connection.recachePreparedStatement(this);
 			return;
 		}
@@ -616,11 +668,14 @@ public class ServerPreparedStatement extends PreparedStatement {
 		this.connection.dumpTestcaseQuery(buf.toString());
 	}
 
-	/**
-	 * @see java.sql.Statement#executeBatch()
-	 */
-	public synchronized int[] executeBatch() throws SQLException {
-		if (this.connection.isReadOnly()) {
+	protected int[] executeBatchSerially(int batchTimeout) throws SQLException {
+		ConnectionImpl locallyScopedConn = this.connection;
+		
+		if (locallyScopedConn == null) {
+			checkClosed();
+		}
+		
+		if (locallyScopedConn.isReadOnly()) {
 			throw SQLError.createSQLException(Messages
 					.getString("ServerPreparedStatement.2") //$NON-NLS-1$
 					+ Messages.getString("ServerPreparedStatement.3"), //$NON-NLS-1$
@@ -629,7 +684,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 		checkClosed();
 
-		synchronized (this.connection.getMutex()) {
+		synchronized (locallyScopedConn.getMutex()) {
 			clearWarnings();
 
 			// Store this for later, we're going to 'swap' them out
@@ -657,77 +712,97 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 					BindValue[] previousBindValuesForBatch = null;
 
-					for (commandIndex = 0; commandIndex < nbrCommands; commandIndex++) {
-						Object arg = this.batchedArgs.get(commandIndex);
-
-						if (arg instanceof String) {
-							updateCounts[commandIndex] = executeUpdate((String) arg);
-						} else {
-							this.parameterBindings = ((BatchedBindValues) arg).batchedParameterValues;
-
-							try {
-								// We need to check types each time, as
-								// the user might have bound different
-								// types in each addBatch()
-
-								if (previousBindValuesForBatch != null) {
-									for (int j = 0; j < this.parameterBindings.length; j++) {
-										if (this.parameterBindings[j].bufferType != previousBindValuesForBatch[j].bufferType) {
-											this.sendTypesToServer = true;
-
-											break;
-										}
-									}
-								}
-
+					CancelTask timeoutTask = null;
+					
+					try {
+						if (locallyScopedConn.getEnableQueryTimeouts() &&
+								batchTimeout != 0
+								&& locallyScopedConn.versionMeetsMinimum(5, 0, 0)) {
+							timeoutTask = new CancelTask(this);
+							ConnectionImpl.getCancelTimer().schedule(timeoutTask,
+									batchTimeout);
+						}
+						
+						for (commandIndex = 0; commandIndex < nbrCommands; commandIndex++) {
+							Object arg = this.batchedArgs.get(commandIndex);
+	
+							if (arg instanceof String) {
+								updateCounts[commandIndex] = executeUpdate((String) arg);
+							} else {
+								this.parameterBindings = ((BatchedBindValues) arg).batchedParameterValues;
+	
 								try {
-									updateCounts[commandIndex] = executeUpdate(false, true);
-								} finally {
-									previousBindValuesForBatch = this.parameterBindings;
-								}
-
-								if (this.retrieveGeneratedKeys) {
-									java.sql.ResultSet rs = null;
-
-									try {
-										// we don't want to use our version,
-										// because we've altered the behavior of
-										// ours to support batch updates
-										// (catch-22)
-										// Ideally, what we need here is
-										// super.super.getGeneratedKeys()
-										// but that construct doesn't exist in
-										// Java, so that's why there's
-										// this kludge.
-										rs = getGeneratedKeysInternal();
-
-										while (rs.next()) {
-											this.batchedGeneratedKeys
-													.add(new ByteArrayRow(new byte[][] { rs
-															.getBytes(1) }));
-										}
-									} finally {
-										if (rs != null) {
-											rs.close();
+									// We need to check types each time, as
+									// the user might have bound different
+									// types in each addBatch()
+	
+									if (previousBindValuesForBatch != null) {
+										for (int j = 0; j < this.parameterBindings.length; j++) {
+											if (this.parameterBindings[j].bufferType != previousBindValuesForBatch[j].bufferType) {
+												this.sendTypesToServer = true;
+	
+												break;
+											}
 										}
 									}
-								}
-							} catch (SQLException ex) {
-								updateCounts[commandIndex] = EXECUTE_FAILED;
-
-								if (this.continueBatchOnError) {
-									sqlEx = ex;
-								} else {
-									int[] newUpdateCounts = new int[commandIndex];
-									System.arraycopy(updateCounts, 0,
-											newUpdateCounts, 0, commandIndex);
-
-									throw new java.sql.BatchUpdateException(ex
-											.getMessage(), ex.getSQLState(), ex
-											.getErrorCode(), newUpdateCounts);
+	
+									try {
+										updateCounts[commandIndex] = executeUpdate(false, true);
+									} finally {
+										previousBindValuesForBatch = this.parameterBindings;
+									}
+	
+									if (this.retrieveGeneratedKeys) {
+										java.sql.ResultSet rs = null;
+	
+										try {
+											// we don't want to use our version,
+											// because we've altered the behavior of
+											// ours to support batch updates
+											// (catch-22)
+											// Ideally, what we need here is
+											// super.super.getGeneratedKeys()
+											// but that construct doesn't exist in
+											// Java, so that's why there's
+											// this kludge.
+											rs = getGeneratedKeysInternal();
+	
+											while (rs.next()) {
+												this.batchedGeneratedKeys
+														.add(new ByteArrayRow(new byte[][] { rs
+																.getBytes(1) }));
+											}
+										} finally {
+											if (rs != null) {
+												rs.close();
+											}
+										}
+									}
+								} catch (SQLException ex) {
+									updateCounts[commandIndex] = EXECUTE_FAILED;
+									
+									if (this.continueBatchOnError && 
+											!(ex instanceof MySQLTimeoutException) && 
+											!(ex instanceof MySQLStatementCancelledException)) {
+										sqlEx = ex;
+									} else {
+										int[] newUpdateCounts = new int[commandIndex];
+										System.arraycopy(updateCounts, 0,
+												newUpdateCounts, 0, commandIndex);
+			
+										throw new java.sql.BatchUpdateException(ex
+												.getMessage(), ex.getSQLState(), ex
+												.getErrorCode(), newUpdateCounts);
+									}
 								}
 							}
 						}
+					} finally {
+						if (timeoutTask != null) {
+							timeoutTask.cancel();
+						}
+						
+						resetCancelledState();
 					}
 
 					if (sqlEx != null) {
@@ -801,6 +876,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 						.toString());
 			}
 
+			sqlEx.initCause(ex);
+			
 			throw sqlEx;
 		}
 	}
@@ -877,7 +954,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 		if (bindValue.isNull) {
 			return null;
 		} else if (bindValue.isLongData) {
-			throw new NotImplemented();
+			throw SQLError.notImplemented();
 		} else {
 			if (this.outByteBuffer == null) {
 				this.outByteBuffer = new Buffer(this.connection
@@ -1023,6 +1100,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 		} catch (Exception ex) {
 			this.invalidationException = SQLError.createSQLException(ex.toString(),
 					SQLError.SQL_STATE_GENERAL_ERROR);
+			this.invalidationException.initCause(ex);
 		}
 
 		if (this.invalidationException != null) {
@@ -1333,7 +1411,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 				this.connection.incrementNumberOfPreparedExecutes();
 
 				if (this.profileSQL) {
-					this.eventSink = ProfileEventSink
+					this.eventSink = ProfilerEventHandlerFactory
 							.getInstance(this.connection);
 
 					this.eventSink.consumeEvent(new ProfilerEvent(
@@ -1611,8 +1689,11 @@ public class ServerPreparedStatement extends PreparedStatement {
 			} catch (SQLException sqlEx) {
 				throw sqlEx;
 			} catch (Exception ex) {
-				throw SQLError.createSQLException(ex.toString(),
+				SQLException sqlEx = SQLError.createSQLException(ex.toString(),
 						SQLError.SQL_STATE_GENERAL_ERROR);
+				sqlEx.initCause(ex);
+				
+				throw sqlEx;
 			} finally {
 				mysql.clearInputStream();
 			}
@@ -1623,7 +1704,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 * @see java.sql.PreparedStatement#setArray(int, java.sql.Array)
 	 */
 	public void setArray(int i, Array x) throws SQLException {
-		throw new NotImplemented();
+		throw SQLError.notImplemented();
 	}
 
 	/**
@@ -1978,7 +2059,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 * @see java.sql.PreparedStatement#setRef(int, java.sql.Ref)
 	 */
 	public void setRef(int i, Ref x) throws SQLException {
-		throw new NotImplemented();
+		throw SQLError.notImplemented();
 	}
 
 	/**
@@ -2075,15 +2156,19 @@ public class ServerPreparedStatement extends PreparedStatement {
 			BindValue binding = getBinding(parameterIndex, false);
 			setType(binding, MysqlDefs.FIELD_TYPE_TIME);
 
-			Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
-			
-			synchronized (sessionCalendar) {
-				binding.value = TimeUtil.changeTimezone(this.connection, 
-						sessionCalendar,
-						targetCalendar,
-						x, tz,
-						this.connection.getServerTimezoneTZ(), 
-						rollForward);
+			if (!this.useLegacyDatetimeCode) {
+				binding.value = x;
+			} else {
+				Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
+				
+				synchronized (sessionCalendar) {
+					binding.value = TimeUtil.changeTimezone(this.connection, 
+							sessionCalendar,
+							targetCalendar,
+							x, tz,
+							this.connection.getServerTimezoneTZ(), 
+							rollForward);
+				}
 			}
 			
 			binding.isNull = false;
@@ -2137,21 +2222,25 @@ public class ServerPreparedStatement extends PreparedStatement {
 			BindValue binding = getBinding(parameterIndex, false);
 			setType(binding, MysqlDefs.FIELD_TYPE_DATETIME);
 
-			Calendar sessionCalendar = this.connection.getUseJDBCCompliantTimezoneShift() ?
-					this.connection.getUtcCalendar() : 
-						getCalendarInstanceForSessionOrNew();
-			
-			synchronized (sessionCalendar) {
-				binding.value = TimeUtil.changeTimezone(this.connection, 
-						sessionCalendar,
-						targetCalendar,
-						x, tz,
-						this.connection.getServerTimezoneTZ(), 
-						rollForward);
+			if (!this.useLegacyDatetimeCode) {
+				binding.value = x;
+			} else {
+				Calendar sessionCalendar = this.connection.getUseJDBCCompliantTimezoneShift() ?
+						this.connection.getUtcCalendar() : 
+							getCalendarInstanceForSessionOrNew();
+				
+				synchronized (sessionCalendar) {
+					binding.value = TimeUtil.changeTimezone(this.connection, 
+							sessionCalendar,
+							targetCalendar,
+							x, tz,
+							this.connection.getServerTimezoneTZ(), 
+							rollForward);
+				}
+				
+				binding.isNull = false;
+				binding.isLongData = false;
 			}
-			
-			binding.isNull = false;
-			binding.isLongData = false;
 		}
 	}
 
@@ -2186,7 +2275,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 			throws SQLException {
 		checkClosed();
 
-		throw new NotImplemented();
+		throw SQLError.notImplemented();
 	}
 
 	/**
@@ -2248,7 +2337,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 			case MysqlDefs.FIELD_TYPE_DATE:
 			case MysqlDefs.FIELD_TYPE_DATETIME:
 			case MysqlDefs.FIELD_TYPE_TIMESTAMP:
-				storeDateTime(packet, (java.util.Date) value, mysql);
+				storeDateTime(packet, (java.util.Date) value, mysql, bindValue.bufferType);
 				return;
 			case MysqlDefs.FIELD_TYPE_VAR_STRING:
 			case MysqlDefs.FIELD_TYPE_STRING:
@@ -2279,10 +2368,22 @@ public class ServerPreparedStatement extends PreparedStatement {
 		}
 	}
 
-	private void storeDataTime412AndOlder(Buffer intoBuf, java.util.Date dt)
+	private void storeDateTime412AndOlder(Buffer intoBuf, java.util.Date dt, int bufferType)
 			throws SQLException {
 		
-		Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
+		Calendar sessionCalendar = null;
+		
+		if (!this.useLegacyDatetimeCode) {
+			if (bufferType == MysqlDefs.FIELD_TYPE_DATE) {
+				sessionCalendar = getDefaultTzCalendar();
+			} else {
+				sessionCalendar = getServerTzCalendar();
+			}
+		} else {
+			sessionCalendar = (dt instanceof Timestamp && 
+				this.connection.getUseJDBCCompliantTimezoneShift()) ? 
+				this.connection.getUtcCalendar() : getCalendarInstanceForSessionOrNew();
+		}
 		
 		synchronized (sessionCalendar) {
 			java.util.Date oldTime = sessionCalendar.getTime();
@@ -2319,25 +2420,34 @@ public class ServerPreparedStatement extends PreparedStatement {
 		}
 	}
 
-	private void storeDateTime(Buffer intoBuf, java.util.Date dt, MysqlIO mysql)
+	private void storeDateTime(Buffer intoBuf, java.util.Date dt, MysqlIO mysql, int bufferType)
 			throws SQLException {
 		if (this.connection.versionMeetsMinimum(4, 1, 3)) {
-			storeDateTime413AndNewer(intoBuf, dt);
+			storeDateTime413AndNewer(intoBuf, dt, bufferType);
 		} else {
-			storeDataTime412AndOlder(intoBuf, dt);
+			storeDateTime412AndOlder(intoBuf, dt, bufferType);
 		}
 	}
 
-	private void storeDateTime413AndNewer(Buffer intoBuf, java.util.Date dt)
+	private void storeDateTime413AndNewer(Buffer intoBuf, java.util.Date dt, int bufferType)
 			throws SQLException {
-		Calendar sessionCalendar = (dt instanceof Timestamp && 
+		Calendar sessionCalendar = null;
+		
+		if (!this.useLegacyDatetimeCode) {
+			if (bufferType == MysqlDefs.FIELD_TYPE_DATE) {
+				sessionCalendar = getDefaultTzCalendar();
+			} else {
+				sessionCalendar = getServerTzCalendar();
+			}
+		} else {
+			sessionCalendar = (dt instanceof Timestamp && 
 				this.connection.getUseJDBCCompliantTimezoneShift()) ? 
 				this.connection.getUtcCalendar() : getCalendarInstanceForSessionOrNew();
+		}
 		
 		synchronized (sessionCalendar) {
 			java.util.Date oldTime = sessionCalendar.getTime();
-		
-		
+
 			try {
 				sessionCalendar.setTime(dt);
 				
@@ -2389,6 +2499,26 @@ public class ServerPreparedStatement extends PreparedStatement {
 		}
 	}
 
+	private Calendar getServerTzCalendar() {
+		synchronized (this) {
+			if (serverTzCalendar == null) {
+				serverTzCalendar = new GregorianCalendar(this.connection.getServerTimezoneTZ());
+			}
+			
+			return this.serverTzCalendar;
+		}
+	}
+	
+	private Calendar getDefaultTzCalendar() {
+		synchronized (this) {
+			if (defaultTzCalendar == null) {
+				defaultTzCalendar = new GregorianCalendar(TimeZone.getDefault());
+			}
+			
+			return this.defaultTzCalendar;
+		}
+	}
+
 	//
 	// TO DO: Investigate using NIO to do this faster
 	//
@@ -2469,9 +2599,12 @@ public class ServerPreparedStatement extends PreparedStatement {
 						null);
 			}
 		} catch (IOException ioEx) {
-			throw SQLError.createSQLException(Messages
+			SQLException sqlEx = SQLError.createSQLException(Messages
 					.getString("ServerPreparedStatement.24") //$NON-NLS-1$
 					+ ioEx.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
+			sqlEx.initCause(ioEx);
+			
+			throw sqlEx;
 		} finally {
 			if (this.connection.getAutoClosePStmtStreams()) {
 				if (inStream != null) {
@@ -2536,9 +2669,12 @@ public class ServerPreparedStatement extends PreparedStatement {
 						null);
 			}
 		} catch (IOException ioEx) {
-			throw SQLError.createSQLException(Messages
+			SQLException sqlEx = SQLError.createSQLException(Messages
 					.getString("ServerPreparedStatement.25") //$NON-NLS-1$
 					+ ioEx.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
+			sqlEx.initCause(ioEx);
+			
+			throw sqlEx;
 		} finally {
 			if (this.connection.getAutoClosePStmtStreams()) {
 				if (inStream != null) {
@@ -2576,5 +2712,184 @@ public class ServerPreparedStatement extends PreparedStatement {
 		return serverStatementId;
 	}
 
+	public synchronized boolean canRewriteAsMultivalueInsertStatement() {
+		if (!super.canRewriteAsMultivalueInsertStatement()) {
+			return false;
+		}
+	
+		BindValue[] currentBindValues = null;
+		BindValue[] previousBindValues = null;
+	
+		int nbrCommands = this.batchedArgs.size();
+	
+		// Can't have type changes between sets of bindings for this to work...
+	
+		for (int commandIndex = 0; commandIndex < nbrCommands; commandIndex++) {
+			Object arg = this.batchedArgs.get(commandIndex);
+	
+			if (!(arg instanceof String)) {
+	
+				currentBindValues = ((BatchedBindValues) arg).batchedParameterValues;
+	
+				// We need to check types each time, as
+				// the user might have bound different
+				// types in each addBatch()
+	
+				if (previousBindValues != null) {
+					for (int j = 0; j < this.parameterBindings.length; j++) {
+						if (currentBindValues[j].bufferType != previousBindValues[j].bufferType) {
+							return false;
+						}
+					}
+				}
+			}
+		}
+		
+		return true;
+	}
+
+	/** 
+	 *  Computes the maximum parameter set size, and entire batch size given 
+	 *  the number of arguments in the batch.
+	 */
+	protected long[] computeMaxParameterSetSizeAndBatchSize(int numBatchedArgs) {
+		long sizeOfEntireBatch = 1 + /* com_execute */ + 4 /* stmt id */ + 1 /* flags */ + 4 /* batch count padding */; 
+		long maxSizeOfParameterSet = 0;
+		
+		for (int i = 0; i < numBatchedArgs; i++) {
+			BindValue[] paramArg = ((BatchedBindValues) this.batchedArgs.get(i)).batchedParameterValues;
+	
+			long sizeOfParameterSet = 0;
+			
+			sizeOfParameterSet += (this.parameterCount + 7) / 8; // for isNull
+			
+			sizeOfParameterSet += this.parameterCount * 2; // have to send types
+			
+			for (int j = 0; j < this.parameterBindings.length; j++) {
+				if (!paramArg[j].isNull) {
+	
+					long size = paramArg[j].getBoundLength();
+					
+					if (paramArg[j].isLongData) {
+						if (size != -1) {
+							sizeOfParameterSet += size;
+						}
+					} else {
+						sizeOfParameterSet += size;
+					}
+				}
+			}
+			
+			sizeOfEntireBatch += sizeOfParameterSet;
+			
+			if (sizeOfParameterSet > maxSizeOfParameterSet) {
+				maxSizeOfParameterSet = sizeOfParameterSet;
+			}
+		}	
+		
+		return new long[] {maxSizeOfParameterSet, sizeOfEntireBatch};
+	}
+
+	protected int setOneBatchedParameterSet(
+			java.sql.PreparedStatement batchedStatement, int batchedParamIndex,
+			Object paramSet) throws SQLException {
+		BindValue[] paramArg = ((BatchedBindValues) paramSet).batchedParameterValues;
+	
+		for (int j = 0; j < paramArg.length; j++) {
+			if (paramArg[j].isNull) {
+				batchedStatement.setNull(batchedParamIndex++, Types.NULL);
+			} else {
+				if (paramArg[j].isLongData) {
+					Object value = paramArg[j].value;
+	
+					if (value instanceof InputStream) {
+						batchedStatement.setBinaryStream(batchedParamIndex++,
+								(InputStream) value,
+								(int) paramArg[j].bindLength);
+					} else {
+						batchedStatement.setCharacterStream(
+								batchedParamIndex++, (Reader) value,
+								(int) paramArg[j].bindLength);
+					}
+				} else {
+	
+					switch (paramArg[j].bufferType) {
+	
+					case MysqlDefs.FIELD_TYPE_TINY:
+						batchedStatement.setByte(batchedParamIndex++,
+								paramArg[j].byteBinding);
+						break;
+					case MysqlDefs.FIELD_TYPE_SHORT:
+						batchedStatement.setShort(batchedParamIndex++,
+								paramArg[j].shortBinding);
+						break;
+					case MysqlDefs.FIELD_TYPE_LONG:
+						batchedStatement.setInt(batchedParamIndex++,
+								paramArg[j].intBinding);
+						break;
+					case MysqlDefs.FIELD_TYPE_LONGLONG:
+						batchedStatement.setLong(batchedParamIndex++,
+								paramArg[j].longBinding);
+						break;
+					case MysqlDefs.FIELD_TYPE_FLOAT:
+						batchedStatement.setFloat(batchedParamIndex++,
+								paramArg[j].floatBinding);
+						break;
+					case MysqlDefs.FIELD_TYPE_DOUBLE:
+						batchedStatement.setDouble(batchedParamIndex++,
+								paramArg[j].doubleBinding);
+						break;
+					case MysqlDefs.FIELD_TYPE_TIME:
+						batchedStatement.setTime(batchedParamIndex++,
+								(Time) paramArg[j].value);
+						break;
+					case MysqlDefs.FIELD_TYPE_DATE:
+						batchedStatement.setDate(batchedParamIndex++,
+								(Date) paramArg[j].value);
+						break;
+					case MysqlDefs.FIELD_TYPE_DATETIME:
+					case MysqlDefs.FIELD_TYPE_TIMESTAMP:
+						batchedStatement.setTimestamp(batchedParamIndex++,
+								(Timestamp) paramArg[j].value);
+						break;
+					case MysqlDefs.FIELD_TYPE_VAR_STRING:
+					case MysqlDefs.FIELD_TYPE_STRING:
+					case MysqlDefs.FIELD_TYPE_VARCHAR:
+					case MysqlDefs.FIELD_TYPE_DECIMAL:
+					case MysqlDefs.FIELD_TYPE_NEW_DECIMAL:
+						Object value = paramArg[j].value;
+	
+						if (value instanceof byte[]) {
+							batchedStatement.setBytes(batchedParamIndex,
+									(byte[]) value);
+						} else {
+							batchedStatement.setString(batchedParamIndex,
+									(String) value);
+						}
+	
+						BindValue asBound = ((ServerPreparedStatement) batchedStatement)
+								.getBinding(
+										batchedParamIndex + 1 /*
+																 * uses 1-based
+																 * offset
+																 */,
+										false);
+						asBound.bufferType = paramArg[j].bufferType;
+	
+						batchedParamIndex++;
+	
+						break;
+					default:
+						throw new IllegalArgumentException(
+								"Unknown type when re-binding parameter into batched statement for parameter index "
+										+ batchedParamIndex);
+					}
+				}
+			}
+		}
+	
+		return batchedParamIndex;
+	}
+
 
 }
