@@ -24,15 +24,8 @@
  */
 package com.mysql.jdbc;
 
-import com.mysql.jdbc.profiler.ProfileEventSink;
-import com.mysql.jdbc.profiler.ProfilerEvent;
-
-import java.io.InputStream;
-import java.io.Reader;
 import java.math.BigDecimal;
-
 import java.sql.SQLException;
-
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -40,6 +33,9 @@ import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
 
+import com.mysql.jdbc.profiler.ProfilerEvent;
+import com.mysql.jdbc.profiler.ProfilerEventHandlerFactory;
+
 /**
  * A result set that is updatable.
  *
@@ -617,7 +613,7 @@ public class UpdatableResultSet extends ResultSetImpl {
 
 		for (int i = 0; i < this.fields.length; i++) {
 		    StringBuffer tableNameBuffer = new StringBuffer();
-		    Map columnNameToIndex = null;
+		    Map updColumnNameToIndex = null;
 
 		    // FIXME: What about no table?
 		    if (this.fields[i].getOriginalTableName() != null) {
@@ -650,7 +646,7 @@ public class UpdatableResultSet extends ResultSetImpl {
 
 	            columnIndicesToTable.put(new Integer(i), fqTableName);
 
-	            columnNameToIndex = getColumnsToIndexMapForTableAndDB(databaseName, tableOnlyName);
+	            updColumnNameToIndex = getColumnsToIndexMapForTableAndDB(databaseName, tableOnlyName);
 	        } else {
 	            String tableOnlyName = this.fields[i].getTableName();
 
@@ -672,7 +668,7 @@ public class UpdatableResultSet extends ResultSetImpl {
 
                     columnIndicesToTable.put(new Integer(i), fqTableName);
 
-                    columnNameToIndex = getColumnsToIndexMapForTableAndDB(this.catalog, tableOnlyName);
+                    updColumnNameToIndex = getColumnsToIndexMapForTableAndDB(this.catalog, tableOnlyName);
 	            }
 	        }
 
@@ -687,8 +683,8 @@ public class UpdatableResultSet extends ResultSetImpl {
 				columnName = this.fields[i].getName();
 			}
 
-			if (columnNameToIndex != null && columnName != null) {
-			    columnNameToIndex.put(columnName, new Integer(i));
+			if (updColumnNameToIndex != null && columnName != null) {
+			    updColumnNameToIndex.put(columnName, new Integer(i));
 			}
 
 			String originalTableName = this.fields[i].getOriginalTableName();
@@ -1177,7 +1173,7 @@ public class UpdatableResultSet extends ResultSetImpl {
 		if (this.useUsageAdvisor) {
 			if ((this.deleter == null) && (this.inserter == null)
 					&& (this.refresher == null) && (this.updater == null)) {
-				this.eventSink = ProfileEventSink.getInstance(this.connection);
+				this.eventSink = ProfilerEventHandlerFactory.getInstance(this.connection);
 
 				String message = Messages.getString("UpdatableResultSet.34"); //$NON-NLS-1$
 
@@ -1416,7 +1412,7 @@ public class UpdatableResultSet extends ResultSetImpl {
 	 * @see DatabaseMetaData#deletesAreDetected
 	 */
 	public synchronized boolean rowDeleted() throws SQLException {
-		throw new NotImplemented();
+		throw SQLError.notImplemented();
 	}
 
 	/**
@@ -1434,7 +1430,7 @@ public class UpdatableResultSet extends ResultSetImpl {
 	 * @see DatabaseMetaData#insertsAreDetected
 	 */
 	public synchronized boolean rowInserted() throws SQLException {
-		throw new NotImplemented();
+		throw SQLError.notImplemented();
 	}
 
 	/**
@@ -1452,7 +1448,7 @@ public class UpdatableResultSet extends ResultSetImpl {
 	 * @see DatabaseMetaData#updatesAreDetected
 	 */
 	public synchronized boolean rowUpdated() throws SQLException {
-		throw new NotImplemented();
+		throw SQLError.notImplemented();
 	}
 
 	/**
