@@ -25,17 +25,15 @@
 package com.mysql.jdbc.jdbc2.optional;
 
 import java.lang.reflect.Constructor;
-import java.sql.Connection;
 import java.sql.SQLException;
 import java.sql.Savepoint;
 import java.sql.Statement;
 import java.util.TimeZone;
 
-import com.mysql.jdbc.ConnectionImpl;
+import com.mysql.jdbc.Connection;
+import com.mysql.jdbc.Extension;
 import com.mysql.jdbc.MysqlErrorNumbers;
-import com.mysql.jdbc.PreparedStatement;
 import com.mysql.jdbc.SQLError;
-import com.mysql.jdbc.ServerPreparedStatement;
 import com.mysql.jdbc.Util;
 import com.mysql.jdbc.log.Log;
 
@@ -61,9 +59,8 @@ import com.mysql.jdbc.log.Log;
  * @see org.gjt.mm.mysql.jdbc2.Connection
  * @see org.gjt.mm.mysql.jdbc2.optional.MysqlPooledConnection
  */
-public class ConnectionWrapper extends WrapperBase implements Connection,
-		com.mysql.jdbc.Connection {
-	protected com.mysql.jdbc.ConnectionImpl mc = null;
+public class ConnectionWrapper extends WrapperBase implements Connection {
+	protected Connection mc = null;
 
 	private MysqlPooledConnection mpc = null;
 
@@ -82,7 +79,7 @@ public class ConnectionWrapper extends WrapperBase implements Connection,
 						"com.mysql.jdbc.jdbc2.optional.JDBC4ConnectionWrapper")
 						.getConstructor(
 								new Class[] { MysqlPooledConnection.class,
-										ConnectionImpl.class, Boolean.TYPE });
+										Connection.class, Boolean.TYPE });
 			} catch (SecurityException e) {
 				throw new RuntimeException(e);
 			} catch (NoSuchMethodException e) {
@@ -97,7 +94,7 @@ public class ConnectionWrapper extends WrapperBase implements Connection,
 
 	protected static ConnectionWrapper getInstance(
 			MysqlPooledConnection mysqlPooledConnection,
-			ConnectionImpl mysqlConnection, boolean forXa) throws SQLException {
+			Connection mysqlConnection, boolean forXa) throws SQLException {
 		if (!Util.isJdbc4()) {
 			return new ConnectionWrapper(mysqlPooledConnection,
 					mysqlConnection, forXa);
@@ -121,7 +118,7 @@ public class ConnectionWrapper extends WrapperBase implements Connection,
 	 *             if an error occurs.
 	 */
 	public ConnectionWrapper(MysqlPooledConnection mysqlPooledConnection,
-			ConnectionImpl mysqlConnection, boolean forXa) throws SQLException {
+			Connection mysqlConnection, boolean forXa) throws SQLException {
 		this.mpc = mysqlPooledConnection;
 		this.mc = mysqlConnection;
 		this.closed = false;
@@ -731,7 +728,7 @@ public class ConnectionWrapper extends WrapperBase implements Connection,
 		checkClosed();
 
 		try {
-			PreparedStatementWrapper.getInstance(this, this.mpc, this.mc
+			return PreparedStatementWrapper.getInstance(this, this.mpc, this.mc
 					.prepareStatement(arg0, arg1));
 		} catch (SQLException sqlException) {
 			checkAndFireConnectionError(sqlException);
@@ -748,7 +745,7 @@ public class ConnectionWrapper extends WrapperBase implements Connection,
 		checkClosed();
 
 		try {
-			PreparedStatementWrapper.getInstance(this, this.mpc, this.mc
+			return PreparedStatementWrapper.getInstance(this, this.mpc, this.mc
 					.prepareStatement(arg0, arg1));
 		} catch (SQLException sqlException) {
 			checkAndFireConnectionError(sqlException);
@@ -765,7 +762,7 @@ public class ConnectionWrapper extends WrapperBase implements Connection,
 		checkClosed();
 
 		try {
-			PreparedStatementWrapper.getInstance(this, this.mpc, this.mc
+			return PreparedStatementWrapper.getInstance(this, this.mpc, this.mc
 					.prepareStatement(arg0, arg1));
 		} catch (SQLException sqlException) {
 			checkAndFireConnectionError(sqlException);
@@ -854,7 +851,7 @@ public class ConnectionWrapper extends WrapperBase implements Connection,
 			}
 
 			if (fireClosedEvent) {
-				this.mpc.callListener(
+				this.mpc.callConnectionEventListeners(
 						MysqlPooledConnection.CONNECTION_CLOSED_EVENT, null);
 			}
 
@@ -877,7 +874,7 @@ public class ConnectionWrapper extends WrapperBase implements Connection,
 		return this.mc.isInGlobalTx();
 	}
 
-	protected void setInGlobalTx(boolean flag) {
+	public void setInGlobalTx(boolean flag) {
 		this.mc.setInGlobalTx(flag);
 	}
 
@@ -2497,4 +2494,48 @@ public class ConnectionWrapper extends WrapperBase implements Connection,
 	public boolean useUnbufferedInput() {
 		return this.mc.useUnbufferedInput();
 	}
+
+	public void initializeExtension(Extension ex) throws SQLException {
+		this.mc.initializeExtension(ex);
+	}
+
+	public String getProfilerEventHandler() {
+		return this.mc.getProfilerEventHandler();
+	}
+
+	public void setProfilerEventHandler(String handler) {
+		this.mc.setProfilerEventHandler(handler);
+	}
+
+	public boolean getVerifyServerCertificate() {
+		return this.mc.getVerifyServerCertificate();
+	}
+
+	public void setVerifyServerCertificate(boolean flag) {
+		this.mc.setVerifyServerCertificate(flag);
+	}
+
+	public boolean getUseLegacyDatetimeCode() {
+		return this.mc.getUseLegacyDatetimeCode();
+	}
+
+	public void setUseLegacyDatetimeCode(boolean flag) {
+		this.mc.setUseLegacyDatetimeCode(flag);
+	}
+
+	public int getSelfDestructOnPingMaxOperations() {
+		return this.mc.getSelfDestructOnPingMaxOperations();
+	}
+
+	public int getSelfDestructOnPingSecondsLifetime() {
+		return this.mc.getSelfDestructOnPingSecondsLifetime();
+	}
+
+	public void setSelfDestructOnPingMaxOperations(int maxOperations) {
+		this.mc.setSelfDestructOnPingMaxOperations(maxOperations);
+	}
+
+	public void setSelfDestructOnPingSecondsLifetime(int seconds) {
+		this.mc.setSelfDestructOnPingSecondsLifetime(seconds);
+	}
 }
