@@ -24,37 +24,28 @@
  */
 package com.mysql.jdbc.jdbc2.optional;
 
-import com.mysql.jdbc.SQLError;
-import com.mysql.jdbc.Util;
-import com.mysql.jdbc.exceptions.NotYetImplementedException;
-
 import java.io.InputStream;
 import java.io.Reader;
-
 import java.lang.reflect.Constructor;
 import java.math.BigDecimal;
-
 import java.net.URL;
-
 import java.sql.Array;
 import java.sql.Blob;
 import java.sql.Clob;
 import java.sql.Date;
-//import java.sql.NClob;
 import java.sql.ParameterMetaData;
 import java.sql.PreparedStatement;
 import java.sql.Ref;
 import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
-import java.sql.Statement;
-//import java.sql.RowId;
 import java.sql.SQLException;
-//import java.sql.SQLXML;
 import java.sql.Time;
 import java.sql.Timestamp;
-
 import java.util.Calendar;
 
+import com.mysql.jdbc.SQLError;
+import com.mysql.jdbc.Util;
+
 /**
  * Wraps prepared statements so that errors can be reported correctly to
  * ConnectionEventListeners.
@@ -1226,10 +1217,10 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 //	}
 //
 //	public boolean isWrapperFor(Class arg0) throws SQLException {
-//		throw new NotYetImplementedException();
+//		throw SQLError.notImplemented();
 //	}
 //
 //	public Object unwrap(Class arg0) throws SQLException {
-//		throw new NotYetImplementedException();
+//		throw SQLError.notImplemented();
 //	}
 }
