@@ -24,18 +24,18 @@
  */
 package com.mysql.jdbc.log;
 
+import java.util.Date;
+
 import com.mysql.jdbc.Util;
 import com.mysql.jdbc.profiler.ProfilerEvent;
 
-import java.util.Date;
-
 /**
  * Provides logging facilities for those platforms that don't have built-in
  * facilities. Simply logs messages to STDERR.
  * 
  * @author Mark Matthews
  * 
- * @version $Id: StandardLogger.java 4596 2005-11-22 21:12:36Z mmatthews $
+ * @version $Id: StandardLogger.java 6702 2007-12-19 14:54:25Z mmatthews $
  */
 public class StandardLogger implements Log {
 	private static final int FATAL = 0;
@@ -61,7 +61,7 @@ public class StandardLogger implements Log {
 	 *            the name of the configuration to use -- ignored
 	 */
 	public StandardLogger(String name) {
-		this(name, true);
+		this(name, false);
 	}
 
 	public StandardLogger(String name, boolean logLocationInfo) {
