@@ -26,7 +26,6 @@ package testsuite.regression;
 
 import java.io.ByteArrayOutputStream;
 import java.io.PrintStream;
-import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.sql.Connection;
@@ -36,6 +35,7 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Locale;
@@ -50,6 +50,7 @@ import com.mysql.jdbc.Driver;
 import com.mysql.jdbc.NonRegisteringDriver;
 import com.mysql.jdbc.ReplicationConnection;
 import com.mysql.jdbc.ReplicationDriver;
+import com.mysql.jdbc.integration.jboss.MysqlValidConnectionChecker;
 import com.mysql.jdbc.log.StandardLogger;
 
 /**
@@ -1513,7 +1514,13 @@ public class ConnectionRegressionTest extends BaseTestCase {
 													.getClass()
 													.getName()
 													.equals(
-															"com.mysql.jdbc.NotImplemented")) {
+															"com.mysql.jdbc.NotImplemented")
+											&& !invokeEx
+											.getCause()
+											.getClass()
+											.getName()
+											.equals(
+													"java.sql.SQLFeatureNotSupportedException")) {
 										throw invokeEx;
 									}
 								}
@@ -2161,28 +2168,65 @@ public class ConnectionRegressionTest extends BaseTestCase {
 		}
 	}
 	
+	public void testBug29106() throws Exception {
+		ClassLoader cl = Thread.currentThread().getContextClassLoader(); 
+		Class checkerClass = cl.loadClass("com.mysql.jdbc.integration.jboss.MysqlValidConnectionChecker");
+		((MysqlValidConnectionChecker)checkerClass.newInstance()).isValidConnection(this.conn);
+	}
+	
 	public void testBug29852() throws Exception {
     	Connection lbConn = getLoadBalancedConnection();
     	assertTrue(!lbConn.getClass().getName().startsWith("com.mysql.jdbc"));
     	lbConn.close();
     }
 
-	private Connection getLoadBalancedConnection() throws SQLException {
+	private Connection getLoadBalancedConnection(int badHostLocation, 
+			String badHost, Properties props) throws SQLException {
 		int indexOfHostStart = dbUrl.indexOf("://") + 3;
     	int indexOfHostEnd = dbUrl.indexOf("/", indexOfHostStart);
     	
-    	String backHalf = dbUrl.substring(indexOfHostStart, indexOfHostEnd);
+    	String firstHost = dbUrl.substring(indexOfHostStart, indexOfHostEnd);
     	
-    	if (backHalf.length() == 0) {
-    		backHalf = "localhost:3306";
+    	if (firstHost.length() == 0) {
+    		firstHost = "localhost:3306";
     	}
     	
     	String dbAndConfigs = dbUrl.substring(indexOfHostEnd);
     	
-    	Connection lbConn = DriverManager.getConnection("jdbc:mysql:loadbalance://" + backHalf + "," + backHalf + dbAndConfigs);
-		return lbConn;
+    	if (badHost != null) {
+    		badHost = badHost + ",";
+    	}
+    	
+    	String hostsString = null;
+    	
+    	switch (badHostLocation) {
+    	case 1:
+    		hostsString = badHost + firstHost;
+    		break;
+    	case 2:
+    		hostsString = firstHost + "," + badHost + firstHost;
+    		break;
+    	case 3:
+    		hostsString = firstHost + "," + badHost;
+    		break;
+    	default:
+    			throw new IllegalArgumentException();
+    	}
+    	
+    	Connection lbConn = DriverManager.getConnection("jdbc:mysql:loadbalance://" + hostsString + dbAndConfigs, props);
+		
+    	return lbConn;
 	}
 	
+	private Connection getLoadBalancedConnection() throws SQLException {
+		return getLoadBalancedConnection(1, "", null);
+	}
+	
+	private Connection getLoadBalancedConnection(Properties props) throws SQLException {
+		return getLoadBalancedConnection(1, "", props);
+	}
+	
+	
 	/**
 	 * Test of a new feature to fix BUG 22643, specifying a
 	 * "validation query" in your connection pool that starts
@@ -2249,4 +2293,104 @@ public class ConnectionRegressionTest extends BaseTestCase {
 			closeMemberJDBCResources();
 		}
 	}
-}
+	
+	public void testBug31053() throws Exception {
+		Properties props = new Properties();
+		props.setProperty("connectTimeout", "2000");
+		props.setProperty("loadBalanceStrategy", "random");
+		
+		Connection lbConn = getLoadBalancedConnection(2, "localhost:23", props);
+		
+		lbConn.setAutoCommit(false);
+		
+		for (int i = 0; i < 10; i++) {
+			lbConn.commit();
+		}
+	}
+	
+	public void testBug32877() throws Exception {
+		Properties props = new Properties();
+		props.setProperty("connectTimeout", "2000");
+		props.setProperty("loadBalanceStrategy", "bestResponseTime");
+		
+		Connection lbConn = getLoadBalancedConnection(1, "localhost:23", props);
+		
+		lbConn.setAutoCommit(false);
+		
+		long begin = System.currentTimeMillis();
+		
+		for (int i = 0; i < 4; i++) {
+			lbConn.commit();
+		}
+		
+		assertTrue(System.currentTimeMillis() - begin < 10000);
+	}
+	
+	/**
+	 * Tests fix for BUG#33734 - NullPointerException when using client-side
+	 * prepared statements and enabling caching of prepared statements (only present
+	 * in nightly builds of 5.1).
+	 * 
+	 * @throws Exception
+	 */
+	public void testBug33734() throws Exception {
+		Connection testConn = getConnectionWithProps("cachePrepStmts=true,useServerPrepStmts=false");
+		try {
+			testConn.prepareStatement("SELECT 1");
+		} finally {
+			testConn.close();
+		}
+	}
+	
+	/** 34703 [NEW]: isValild() aborts Connection on timeout */
+	
+	public void testBug34703() throws Exception {
+		if (!com.mysql.jdbc.Util.isJdbc4()) {
+			return;
+		}
+		
+		Method isValid = java.sql.Connection.class.getMethod("isValid", new Class[] {Integer.TYPE});
+		
+		
+		Connection newConn = getConnectionWithProps((Properties)null);
+		isValid.invoke(newConn, new Object[] {new Integer(1)});
+		Thread.sleep(2000);
+		assertTrue(((Boolean)isValid.invoke(newConn, new Object[] {new Integer(0)})).booleanValue());
+	}
+	
+	public void testBug34937() throws Exception {
+		com.mysql.jdbc.jdbc2.optional.MysqlConnectionPoolDataSource ds = new
+		com.mysql.jdbc.jdbc2.optional.MysqlConnectionPoolDataSource();
+		StringBuffer urlBuf = new StringBuffer();
+		urlBuf.append(getMasterSlaveUrl());
+		urlBuf.append("?");
+		Properties props = getMasterSlaveProps();
+		String key = null;
+		
+		Enumeration keyEnum = props.keys();
+		
+		while (keyEnum.hasMoreElements()) {
+			key = (String)keyEnum.nextElement();
+			urlBuf.append(key);
+			urlBuf.append("=");
+			urlBuf.append(props.get(key));
+			urlBuf.append("&");
+		}
+		
+		String url = urlBuf.toString();
+		url = "jdbc:mysql:replication:" + url.substring(url.indexOf("jdbc:mysql:") + "jdbc:mysql:".length());
+		ds.setURL(url);
+		Connection replConn = ds.getPooledConnection().getConnection();
+		
+		boolean readOnly = false;
+		
+		for (int i = 0; i < 10; i++) {
+			this.rs = replConn.createStatement().executeQuery("SELECT 1");
+			assertTrue(this.rs.next());
+			this.rs = replConn.prepareStatement("SELECT 1").executeQuery();
+			assertTrue(this.rs.next());
+			readOnly = !readOnly;
+			replConn.setReadOnly(readOnly);
+		}	
+	}
+}
\ No newline at end of file
