@@ -39,7 +39,6 @@ import java.util.Properties;
 import testsuite.BaseTestCase;
 
 import com.mysql.jdbc.Driver;
-import com.mysql.jdbc.Field;
 import com.mysql.jdbc.NonRegisteringDriver;
 import com.mysql.jdbc.SQLError;
 
@@ -1961,6 +1960,10 @@ public class MetaDataRegressionTest extends BaseTestCase {
 	 * @throws Exception if the test fails.
 	 */
 	public void testBug27867() throws Exception {
+		if (!versionMeetsMinimum(4, 1)) {
+			return;
+		}
+		
 		try {
 			String gbkColumnName = "\u00e4\u00b8\u00ad\u00e6\u2013\u2021\u00e6\u00b5\u2039\u00e8\u00af\u2022";
 			createTable("ColumnNameEncoding", "(" + "`" + gbkColumnName
@@ -2067,4 +2070,142 @@ public class MetaDataRegressionTest extends BaseTestCase {
 					typeNameToPrecision.get(typeName));
 		}
 	}
+	
+	public void testBug20491() throws Exception {
+		try {
+			String[] fields = { "field1_ae_", "field2_ue_", "field3_oe_",
+			"field4_sz_" };
+			
+			createTable("tst",
+					"(`" + fields[0] + "` int(10) unsigned NOT NULL default '0',"
+							+ "`" + fields[1] + "` varchar(45) default '',"
+							+ "`" + fields[2] + "` varchar(45) default '',"
+							+ "`" + fields[3] + "` varchar(45) default '',"
+							+ "PRIMARY KEY  (`" + fields[0] + "`))");
+
+			// demonstrate that these are all in the Cp1252 encoding
+			
+			for (int i = 0; i < fields.length; i++) {
+				assertEquals(fields[i], new String(fields[i].getBytes("Cp1252"), "Cp1252"));
+			}
+			
+			byte[] asBytes = fields[0].getBytes("utf-8");
+			
+			DatabaseMetaData md = this.conn.getMetaData();
+			
+			this.rs = md.getColumns(null, "%", "tst", "%");
+			
+			int j = 0;
+			
+			while (this.rs.next()) {
+				assertEquals("Wrong column name:" + this.rs.getString(4),
+						fields[j++], this.rs.getString(4));
+			}
+			
+			this.rs.close();
+			
+			this.rs = this.stmt.executeQuery("SELECT * FROM tst");
+			
+			ResultSetMetaData rsmd = this.rs.getMetaData();
+			
+			for (int i = 1; i <= rsmd.getColumnCount(); i++) {
+				assertEquals("Wrong column name:" + rsmd.getColumnName(i),
+						fields[i - 1], rsmd.getColumnName(i));
+			}
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
+	
+	/**
+	 * Tests fix for Bug#33594 - When cursor fetch is enabled,
+	 * wrong metadata is returned from DBMD. 
+	 * 
+	 * The fix is two parts. 
+	 * 
+	 * First, when asking for the first column value
+	 * twice from a cursor-fetched row, the driver didn't re-position,
+	 * and thus the "next" column was returned.
+	 * 
+	 * Second, metadata statements and internal statements the driver
+	 * uses shouldn't use cursor-based fetching at all, so we've
+	 * ensured that internal statements have their fetch size set to "0".
+	 */
+	public void testBug33594() throws Exception {
+		if (!versionMeetsMinimum(5, 0, 7)) {
+			return;
+		}
+
+		try {
+			createTable(
+					"bug33594",
+					"(fid varchar(255) not null primary key, id INT, geom linestring, name varchar(255))");
+
+			Properties props = new Properties();
+			props.put("useInformationSchema", "false");
+			props.put("useCursorFetch", "false");
+			props.put("defaultFetchSize", "100");
+			Connection conn1 = null;
+			try {
+				conn1 = getConnectionWithProps(props);
+				DatabaseMetaData metaData = conn1.getMetaData();
+				this.rs = metaData.getColumns(null, null, "bug33594", null);
+				this.rs.next();
+				assertEquals("bug33594", this.rs.getString("TABLE_NAME"));
+				assertEquals("fid", this.rs.getString("COLUMN_NAME"));
+				assertEquals("VARCHAR", this.rs.getString("TYPE_NAME"));
+				assertEquals("255", this.rs.getString("COLUMN_SIZE"));
+
+				Properties props2 = new Properties();
+				props2.put("useInformationSchema", "false");
+				props2.put("useCursorFetch", "true");
+				props2.put("defaultFetchSize", "100");
+
+				Connection conn2 = null;
+
+				try {
+					conn2 = getConnectionWithProps(props2);
+					DatabaseMetaData metaData2 = conn2.getMetaData();
+					this.rs = metaData2
+							.getColumns(null, null, "bug33594", null);
+					this.rs.next();
+					assertEquals("bug33594", this.rs.getString("TABLE_NAME"));
+					assertEquals("fid", this.rs.getString("COLUMN_NAME"));
+					assertEquals("VARCHAR", this.rs.getString("TYPE_NAME"));
+					assertEquals("255", this.rs.getString("COLUMN_SIZE"));
+					
+					// we should only see one server-side prepared statement, and that's
+					// caused by us going off to ask about the count!
+					assertEquals("1", getSingleIndexedValueWithQuery(conn2, 2,
+							"SHOW SESSION STATUS LIKE 'Com_stmt_prepare'")
+							.toString());
+				} finally {
+					if (conn2 != null) {
+						conn2.close();
+					}
+				}
+			} finally {
+				if (conn1 != null) {
+					conn1.close();
+				}
+			}
+
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
+	
+	public void testBug34194() throws Exception {
+		createTable("bug34194", "(id integer,geom geometry)");
+		try {
+			this.stmt
+					.execute("insert into bug34194 values('1',GeomFromText('POINT(622572.881 5156121.034)'))");
+			this.rs = this.stmt.executeQuery("select * from bug34194");
+			ResultSetMetaData RSMD = this.rs.getMetaData();
+			assertEquals("GEOMETRY", RSMD.getColumnTypeName(2));
+
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
 }
