@@ -36,15 +36,15 @@ import java.util.List;
 import java.util.Properties;
 import java.util.Set;
 
-import com.mysql.jdbc.StringUtils;
-
 import testsuite.BaseTestCase;
 
+import com.mysql.jdbc.StringUtils;
+
 /**
  * Tests DatabaseMetaData methods.
  * 
  * @author Mark Matthews
- * @version $Id: MetadataTest.java 5828 2006-10-05 16:51:31Z mmatthews $
+ * @version $Id: MetadataTest.java 6702 2007-12-19 14:54:25Z mmatthews $
  */
 public class MetadataTest extends BaseTestCase {
 	// ~ Constructors
@@ -592,7 +592,7 @@ public class MetadataTest extends BaseTestCase {
                 this.rs.next();
                 assertEquals("t1", this.rs.getString("TABLE_NAME"));
                 assertEquals("c1", this.rs.getString("COLUMN_NAME"));
-                assertEquals("char", this.rs.getString("TYPE_NAME"));
+                assertEquals("CHAR", this.rs.getString("TYPE_NAME"));
                 assertEquals("1", this.rs.getString("COLUMN_SIZE"));
             } finally {
                 if (conn1 != null) {
